//ObbMesh.cpp
#include "ObbMesh.h"


ObbMesh::ObbMesh() 
{
	m_obb.center = D3DXVECTOR3( 0, 0, 0 );
	D3DXMatrixIdentity( &m_obb.orientation );
	//m_obb.orientation[0] = D3DXVECTOR3( 1, 0, 0 );
	//m_obb.orientation[1] = D3DXVECTOR3( 0, 1, 0 );
	//m_obb.orientation[2] = D3DXVECTOR3( 0, 0, 1 );
	m_obb.extent = D3DXVECTOR3( 1, 1, 1 );
}
//--------------------------------------------------------------------------------------
ObbMesh::~ObbMesh()
{
}
//--------------------------------------------------------------------------------------
OBB& ObbMesh::GetCollision()
{
	return m_obb;
}
//--------------------------------------------------------------------------------------
void ObbMesh::SetTranslate( float x, float y, float z )
{
	m_obb.center.x = x;
	m_obb.center.y = y;
	m_obb.center.z = z;
}
//--------------------------------------------------------------------------------------
void ObbMesh::SetRotate( float xaxis, float yaxis, float zaxis )
{
	//m_obb.orientation[0] = xaxis;
	//m_obb.orientation[1] = yaxis;
	//m_obb.orientation[2] = zaxis;
	D3DXMatrixIdentity( &m_obb.orientation );
	D3DXMatrixRotationYawPitchRoll( &m_obb.orientation, yaxis, xaxis, zaxis );
}
//--------------------------------------------------------------------------------------
void ObbMesh::SetRadius( float hwx, float hwy, float hwz )
{
	assert( hwx >= 0.001f );
	assert( hwy >= 0.001f );
	assert( hwz >= 0.001f );
	m_obb.extent.x = hwx;
	m_obb.extent.y = hwy;
	m_obb.extent.z = hwz;
}
//--------------------------------------------------------------------------------------
void ObbMesh::Render( const LPDIRECT3DDEVICE9 device )
{
	D3DXMATRIX matTrans, mat;
	D3DXMatrixTranslation( &matTrans, m_obb.center.x, m_obb.center.y, m_obb.center.z );
	mat = m_obb.orientation * matTrans;
	device->SetTransform( D3DTS_WORLD, &mat );
	struct Vertex
	{
		D3DXVECTOR3		pos; 
		DWORD			color; 
	};
	int type = ( D3DFVF_XYZ | D3DFVF_DIFFUSE );

	float hwx = m_obb.extent.x; float hwy = m_obb.extent.y;  float hwz = m_obb.extent.z;
	D3DXVECTOR3 pos[8] = {
		D3DXVECTOR3( -hwx, -hwy,  hwz ),
		D3DXVECTOR3(  hwx, -hwy,  hwz ),
		D3DXVECTOR3(  hwx,  hwy,  hwz ),
		D3DXVECTOR3( -hwx,  hwy,  hwz ),
		D3DXVECTOR3( -hwx, -hwy, -hwz ),
		D3DXVECTOR3(  hwx, -hwy, -hwz ),
		D3DXVECTOR3(  hwx,  hwy, -hwz ),
		D3DXVECTOR3( -hwx,  hwy, -hwz ) };

	Vertex vet[24];
	vet[0].pos =  pos[0]; vet[1].pos =  pos[1];      vet[2].pos = pos[1];  vet[3].pos = pos[2];
	vet[4].pos =  pos[2]; vet[5].pos =  pos[3];      vet[6].pos = pos[3];  vet[7].pos = pos[0];
	vet[8].pos =  pos[4]; vet[9].pos =  pos[5];      vet[10].pos = pos[5]; vet[11].pos = pos[6];
	vet[12].pos = pos[6]; vet[13].pos = pos[7];      vet[14].pos = pos[7]; vet[15].pos = pos[4];

	vet[16].pos = pos[0]; vet[17].pos = pos[4];      vet[18].pos = pos[1]; vet[19].pos = pos[5];
	vet[20].pos = pos[2]; vet[21].pos = pos[6];      vet[22].pos = pos[3]; vet[23].pos = pos[7];

	for( int i = 0; i < 24; ++i )
	{
		vet[i].color = m_color;
	}

	device->SetRenderState( D3DRS_LIGHTING, FALSE );
	device->SetFVF( type );
	HRESULT hr = device->DrawPrimitiveUP( D3DPT_LINELIST, 24/2, vet, sizeof(Vertex) );
	device->SetRenderState( D3DRS_LIGHTING, TRUE );

}
//------------------------------------------------------------------------------------------------
void ObbMesh::SetColor( float r, float g, float	 b )
{
	m_color = D3DXCOLOR( r, g, b, 1.0f );
}