//SphereMesh.cpp

#include "SphereMesh.h"

SphereMesh::SphereMesh()
{
	m_pMesh = NULL;
	m_sphere.radius = 1.0f;
	m_sphere.center = D3DXVECTOR3( 1, 1, 1 );
}
//-------------------------------------------------------------------------------------
SphereMesh::~SphereMesh()
{
	if( m_pMesh )
	{
		m_pMesh->Release();
		m_pMesh = NULL;
	}
}
//-------------------------------------------------------------------------------------
Sphere& SphereMesh::GetCollision()
{
	return m_sphere;
}
//-------------------------------------------------------------------------------------
void SphereMesh::SetRadius( float r )
{
	m_sphere.radius = r;
}
//-------------------------------------------------------------------------------------
void SphereMesh::SetTranslate( float x, float y, float z )
{
	m_sphere.center = D3DXVECTOR3( x, y, z );
}
//-------------------------------------------------------------------------------------
void SphereMesh::Render( const LPDIRECT3DDEVICE9 device )
{
	if( m_pMesh == NULL )
		D3DXCreateSphere( device, 1.0f, 20, 10, &m_pMesh, NULL );
	D3DXMATRIX matTrans, matScale, mat;
	D3DXMatrixScaling( &matScale, m_sphere.radius, m_sphere.radius, m_sphere.radius );
	D3DXMatrixTranslation( &matTrans, m_sphere.center.x, m_sphere.center.y, m_sphere.center.z );
	mat = matScale * matTrans;
	device->SetTransform( D3DTS_WORLD, &mat );

    D3DMATERIAL9 mtrl1;   
    ZeroMemory( &mtrl1, sizeof(D3DMATERIAL9) );   
    mtrl1.Diffuse.r = mtrl1.Ambient.r = 1.0f;   
    mtrl1.Diffuse.g = mtrl1.Ambient.g = 1.0f;   
    mtrl1.Diffuse.b = mtrl1.Ambient.b = 1.0f;   
    mtrl1.Diffuse.a = mtrl1.Ambient.a = 1.0f;   
    device->SetMaterial(&mtrl1);   
   
    device->SetRenderState( D3DRS_NORMALIZENORMALS, TRUE );     
    //device->SetRenderState( D3DRS_SHADEMODE, D3DSHADE_GOURAUD );  
	//device->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );
    m_pMesh->DrawSubset(0);   
	//device->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );
}
//-------------------------------------------------------------------------------------
