//collision.cpp
#include "collision.h"

const float EPSILON = 0.001f;

// ǥ迡 ﰢ   ִ ˻Ѵ.
bool IsColinearTriangle( D3DXVECTOR3* p0, D3DXVECTOR3* p1, D3DXVECTOR3* p2, float epsilon )
{
    D3DXVECTOR3 u = *p1 - *p0;  
    D3DXVECTOR3 v = *p2 - *p1;

    D3DXVec3Normalize( &u, &u );
	D3DXVec3Normalize( &v, &v );

	float dot = D3DXVec3Dot( &u, &v );
	
	//p0 --> p1 --> p2  ̳ʽ ϶,  ٲ ̱   dot  -1  false ȯѴ.  
	// ٲ   p0--->p1   p1--->p2϶  p0 p1  ݴ ̴.

	if ( dot >= epsilon )
		return true;
	else
		return false;
}

//ϼ ͳ ģ.
void ConsolidatePolygon( VertexContainer& out, VertexContainer& in )
{
	int    nCount = (int)in.size();
	if( nCount < 3 )
		return;

	int	   prev = nCount - 1;
	int    curr = 0;
	int    next = 1;

    for ( int i=0; i<nCount; i++)
    {
		if( next == nCount )
			next = 0;

		if( IsColinearTriangle( &in[prev], &in[curr], &in[next] ) == false )
			out.push_back( in[curr] );

		prev = i;
		++curr;
		next = curr + 1;
    }
}

//-----------------------------------------------------------------------------------
//Real Time Collision Detection ҽ
// Given point p, return point q on (or in) OBB b, closest to p
#if  0
void ClosestPtPointOBB(Point p, OBB b, Point &q)
{
    Vector d = p - b.c;
    // Start result at center of box; make steps from there
    q = b.c;
    // For each OBB axis...
    for (int i = 0; i < 3; i++) 
	{
        // ...project d onto that axis to get the distance
        // along the axis of d from the box center
        float dist = Dot(d, b.u[i]);
        // If distance farther than the box extents, clamp to the box
        if (dist > b.e[i]) dist = b.e[i];
        if (dist < -b.e[i]) dist = -b.e[i];
        // Step that distance along the axis to get world coordinate
        q += dist * b.u[i];
    }
}

// Returns true if sphere s intersects OBB b, false otherwise.
// The point p on the OBB closest to the sphere center is also returned
int TestSphereOBB(Sphere s, OBB b, Point &p)
{
    // Find point p on OBB closest to sphere center
    ClosestPtPointOBB(s.c, b, p);

    // Sphere and OBB intersect if the (squared) distance from sphere
    // center to point p is less than the (squared) sphere radius
    Vector v = p - s.c;
    return Dot(v, v) <= s.r * s.r;
}
#else
void ClosestPtPointOBB( D3DXVECTOR3 p, OBB b, D3DXVECTOR3 &q )
{
	D3DXVECTOR3 d = p - b.center;
    // Start result at center of box; make steps from there
    q = b.center;

	D3DXMATRIX& mat = b.orientation;
	D3DXVECTOR3	vec[3] = {  D3DXVECTOR3( mat._11, mat._21, mat._31 ),
							D3DXVECTOR3( mat._12, mat._22, mat._32 ),
							D3DXVECTOR3( mat._13, mat._23, mat._33 )  };

    // For each OBB axis...
    for (int i = 0; i < 3; i++) 
	{
        // ...project d onto that axis to get the distance
        // along the axis of d from the box center
        float dist = D3DXVec3Dot( &d, &vec[i] );
        // If distance farther than the box extents, clamp to the box
		if (dist > b.extent[i]) 
			dist = b.extent[i];
		if (dist < -b.extent[i]) 
			dist = -b.extent[i];
        // Step that distance along the axis to get world coordinate
        q += dist * vec[i];
    }
}

// Returns true if sphere s intersects OBB b, false otherwise.
// The point p on the OBB closest to the sphere center is also returned
bool TestSphereOBB( const Sphere& s, const OBB& b, D3DXVECTOR3* p )
{
    // Find point p on OBB closest to sphere center
	ClosestPtPointOBB(s.center, b, *p);

    // Sphere and OBB intersect if the (squared) distance from sphere
    // center to point p is less than the (squared) sphere radius
    D3DXVECTOR3 v = *p - s.center;
    return D3DXVec3Dot( &v, &v) <= ( s.radius * s.radius );
}
#endif
//-----------------------------------------------------------------------------------
bool TestObbAndObb( const OBB& a, OBB& b )
{
#if  0
	//float			ra, rb;
	D3DXMATRIXA16   r, absR;

	for( int row = 0; row < 3; ++row )
		for( int col = 0; col < 3; ++col )
			r[row*4 + col] = D3DXVec3Dot( &a.orientation[col], &b.orientation[row] );	

    // Compute translation vector t
	D3DXVECTOR3 t = b.center - a.center;
    // Bring translation into a's coordinate frame
    t = D3DXVECTOR3( D3DXVec3Dot( &t, &a.orientation[0]),  
					 D3DXVec3Dot( &t, &a.orientation[1]), 
					 D3DXVec3Dot( &t, &a.orientation[2]));


    // Compute common subexpressions. Add in an epsilon term to
    // counteract arithmetic errors when two edges are parallel and
    // their cross product is (near) null (see text for details)
    for (int row = 0; row < 3; row++)
        for (int col = 0; col < 3; col++)
            absR[row*4 + col] = abs(r[row * 4 + col]) + EPSILON;
#endif
#if  0
    // Test axes L = A0, L = A1, L = A2
    for (int i = 0; i < 3; i++) {
        ra = a.e[i];
        rb = b.e[0] * AbsR[i][0] + b.e[1] * AbsR[i][1] + b.e[2] * AbsR[i][2];
        if (Abs(t[i]) > ra + rb) return 0;
    }

    // Test axes L = B0, L = B1, L = B2
    for (int i = 0; i < 3; i++) {
        ra = a.e[0] * AbsR[0][i] + a.e[1] * AbsR[1][i] + a.e[2] * AbsR[2][i];
        rb = b.e[i];
        if (Abs(t[0] * R[0][i] + t[1] * R[1][i] + t[2] * R[2][i]) > ra + rb) return 0;
    }

    // Test axis L = A0 x B0
    ra = a.e[1] * AbsR[2][0] + a.e[2] * AbsR[1][0];
    rb = b.e[1] * AbsR[0][2] + b.e[2] * AbsR[0][1];
    if (Abs(t[2] * R[1][0] - t[1] * R[2][0]) > ra + rb) return 0;

    // Test axis L = A0 x B1
    ra = a.e[1] * AbsR[2][1] + a.e[2] * AbsR[1][1];
    rb = b.e[0] * AbsR[0][2] + b.e[2] * AbsR[0][0];
    if (Abs(t[2] * R[1][1] - t[1] * R[2][1]) > ra + rb) return 0;

    // Test axis L = A0 x B2
    ra = a.e[1] * AbsR[2][2] + a.e[2] * AbsR[1][2];
    rb = b.e[0] * AbsR[0][1] + b.e[1] * AbsR[0][0];
    if (Abs(t[2] * R[1][2] - t[1] * R[2][2]) > ra + rb) return 0;

    // Test axis L = A1 x B0
    ra = a.e[0] * AbsR[2][0] + a.e[2] * AbsR[0][0];
    rb = b.e[1] * AbsR[1][2] + b.e[2] * AbsR[1][1];
    if (Abs(t[0] * R[2][0] - t[2] * R[0][0]) > ra + rb) return 0;

    // Test axis L = A1 x B1
    ra = a.e[0] * AbsR[2][1] + a.e[2] * AbsR[0][1];
    rb = b.e[0] * AbsR[1][2] + b.e[2] * AbsR[1][0];
    if (Abs(t[0] * R[2][1] - t[2] * R[0][1]) > ra + rb) return 0;

    // Test axis L = A1 x B2
    ra = a.e[0] * AbsR[2][2] + a.e[2] * AbsR[0][2];
    rb = b.e[0] * AbsR[1][1] + b.e[1] * AbsR[1][0];
    if (Abs(t[0] * R[2][2] - t[2] * R[0][2]) > ra + rb) return 0;

    // Test axis L = A2 x B0
    ra = a.e[0] * AbsR[1][0] + a.e[1] * AbsR[0][0];
    rb = b.e[1] * AbsR[2][2] + b.e[2] * AbsR[2][1];
    if (Abs(t[1] * R[0][0] - t[0] * R[1][0]) > ra + rb) return 0;

    // Test axis L = A2 x B1
    ra = a.e[0] * AbsR[1][1] + a.e[1] * AbsR[0][1];
    rb = b.e[0] * AbsR[2][2] + b.e[2] * AbsR[2][0];
    if (Abs(t[1] * R[0][1] - t[0] * R[1][1]) > ra + rb) return 0;

    // Test axis L = A2 x B2
    ra = a.e[0] * AbsR[1][2] + a.e[1] * AbsR[0][2];
    rb = b.e[0] * AbsR[2][1] + b.e[1] * AbsR[2][0];
    if (Abs(t[1] * R[0][2] - t[0] * R[1][2]) > ra + rb) return 0;
#endif
    // Since no separating axis found, the OBBs must be intersecting
    return true;

}
//--------------------------------------------------------------------------------------
